%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This MATLAB code was used to calculate the numerical integration of the
% covariance term of Inverse normal combination method in Section 2.2.2, Example 2
% Matlab program written by Shiyang Ma at 2018-12-05
% The matlab version is R2018a
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Define the fine grid of rho and nu
rho_seq=-0.98:0.02:0.98;
v_seq=9:125;
cov = zeros(1,length(rho_seq).*length(v_seq));
tic
for i = 1:length(rho_seq)
    for j = 1:length(v_seq)
        rho=rho_seq(i);
        v=v_seq(j);
        %Inverse function g(W_m) of Inverse normal combination method
        g=@(w) tinv(1-normcdf(w,0,1),v); 
        %The function under the integral is w_r*w_s times the joint density function 
        %of f_{W_r,W_s} pf the Inverse normal combination method, following
        %the covariance terms 
        f=@(x,y) x.*y.*normpdf(x,0,1).*normpdf(y,0,1)./(2.*pi.*sqrt(1-rho^2).*tpdf(g(x),v).*tpdf(g(y),v)).*(v.*(1-rho.^2)./(v.*(1-rho.^2)+g(x).^2+g(y).^2-2.*rho.*g(x).*g(y))).^((v+2)/2); 
        cov(1,length(v_seq).*(i-1)+j)=integral2(f,-8.29,8.29,-8.29,8.29);
    end
end
toc

dlmwrite('Inverse normal numerical integration.csv', cov, 'delimiter', ',', 'precision', 20);  %29.6min 
% Then use R code 'R code_Covariance terms_Inverse normal.R' to calculate the
% polynomial approximation of the covariance term of Inverse normal combination method
% Take 29.6 minutes to get run
