#########################################################################
### R code for Numerical example and Table 1 in MA and MCDERMOTT
### Generalized Multiple Contrast Tests in Dose-Response Studies
### R program written by Shiyang Ma at 2018-12-05
#########################################################################

#Get the biom data from DoseFinding package
library(DoseFinding)

data(biom)
d_biom=biom[,1]
y_biom=biom[,2]

#Calculate contrasts
contrast=function(mu){
   (mu-mean(mu))/norm(mu-mean(mu),type='2')
}
#5 dosage groups
d=c(0,0.05,0.2,0.6,1)

#7 candidate models
c1=contrast(d/(0.2+d)) #Emax
c2=contrast(log(5*d+1)) #L-L   
c3=contrast(d-0.854*d^2) #Q1
c4=contrast(d) #Linear
c5=contrast(exp(d/0.279)) #E1
c6=contrast(exp(d/0.15)) #E2
c7=contrast(d-d^2) #Q2

rho12=sum(c1*c2)/sqrt(sum(c1^2)*sum(c2^2))
rho13=sum(c1*c3)/sqrt(sum(c1^2)*sum(c3^2))
rho14=sum(c1*c4)/sqrt(sum(c1^2)*sum(c4^2))
rho15=sum(c1*c5)/sqrt(sum(c1^2)*sum(c5^2))
rho16=sum(c1*c6)/sqrt(sum(c1^2)*sum(c6^2))
rho17=sum(c1*c7)/sqrt(sum(c1^2)*sum(c7^2))

rho23=sum(c2*c3)/sqrt(sum(c2^2)*sum(c3^2))
rho24=sum(c2*c4)/sqrt(sum(c2^2)*sum(c4^2))
rho25=sum(c2*c5)/sqrt(sum(c2^2)*sum(c5^2))
rho26=sum(c2*c6)/sqrt(sum(c2^2)*sum(c6^2))
rho27=sum(c2*c7)/sqrt(sum(c2^2)*sum(c7^2))

rho34=sum(c3*c4)/sqrt(sum(c3^2)*sum(c4^2))
rho35=sum(c3*c5)/sqrt(sum(c3^2)*sum(c5^2))
rho36=sum(c3*c6)/sqrt(sum(c3^2)*sum(c6^2))
rho37=sum(c3*c7)/sqrt(sum(c3^2)*sum(c7^2))

rho45=sum(c4*c5)/sqrt(sum(c4^2)*sum(c5^2))
rho46=sum(c4*c6)/sqrt(sum(c4^2)*sum(c6^2))
rho47=sum(c4*c7)/sqrt(sum(c4^2)*sum(c7^2))

rho56=sum(c5*c6)/sqrt(sum(c5^2)*sum(c6^2))
rho57=sum(c5*c7)/sqrt(sum(c5^2)*sum(c7^2))

rho67=sum(c6*c7)/sqrt(sum(c6^2)*sum(c7^2))

#Correlation matrix
Sigma=matrix(c(1,rho12, rho13,rho14,rho15, rho16, rho17,
               rho12, 1, rho23, rho24,rho25,  rho26, rho27,
               rho13, rho23, 1, rho34, rho35,  rho36, rho37,
               rho14, rho24, rho34, 1, rho45,  rho46, rho47,
               rho15, rho25, rho35, rho45, 1,  rho56, rho57,
               rho16, rho26, rho36, rho46,  rho56, 1, rho67,
               rho17, rho27, rho37, rho47,  rho57, rho67,1
),nrow=7,ncol=7)

#Sample mean y^bar
ybar1=mean(y_biom[d_biom==0])
ybar2=mean(y_biom[d_biom==0.05])
ybar3=mean(y_biom[d_biom==0.2])
ybar4=mean(y_biom[d_biom==0.6])
ybar5=mean(y_biom[d_biom==1])
ybar=c(ybar1,ybar2,ybar3,ybar4,ybar5)

#Contrast estimates in Table 1
ce1=sum(c1*ybar)
ce2=sum(c2*ybar) 
ce3=sum(c3*ybar)
ce4=sum(c4*ybar)
ce5=sum(c5*ybar)
ce6=sum(c6*ybar)
ce7=sum(c7*ybar)
ce=c(ce1,ce2,ce3,ce4,ce5,ce6,ce7)
round(ce,digits=4)

#T-value in Table 1
df=100-5
S2=(sum((y_biom[d_biom==0]-ybar1)^2)+sum((y_biom[d_biom==0.05]-ybar2)^2)+
   sum((y_biom[d_biom==0.2]-ybar3)^2)+sum((y_biom[d_biom==0.6]-ybar4)^2)+
   sum((y_biom[d_biom==1]-ybar5)^2))/df
t1=ce1/sqrt(S2*sum(c1^2)/20)
t2=ce2/sqrt(S2*sum(c2^2)/20)
t3=ce3/sqrt(S2*sum(c3^2)/20)
t4=ce4/sqrt(S2*sum(c4^2)/20)
t5=ce5/sqrt(S2*sum(c5^2)/20)
t6=ce6/sqrt(S2*sum(c6^2)/20)
t7=ce7/sqrt(S2*sum(c7^2)/20)
t=c(t1,t2,t3,t4,t5,t6,t7)
round(t,digits=4)

#Raw p values in Table 1
p1=1-pt(t1,df=95)
p2=1-pt(t2,df=95)
p3=1-pt(t3,df=95)
p4=1-pt(t4,df=95)
p5=1-pt(t5,df=95)
p6=1-pt(t6,df=95)
p7=1-pt(t7,df=95)
p=c(p1,p2,p3,p4,p5,p6,p7)
round(p,digits=4)

#Adjusted p-values of MCP-Mod in Table 1
library(mvtnorm)
#7 adjusted p-values
set.seed(12345)
p1_adj_MCPMod=1-pmvt(lower=rep(-Inf,7),upper =rep(t1,7), corr=Sigma, df=95)[1]
p2_adj_MCPMod=1-pmvt(lower=rep(-Inf,7),upper =rep(t2,7), corr=Sigma, df=95)[1]
p3_adj_MCPMod=1-pmvt(lower=rep(-Inf,7),upper =rep(t3,7), corr=Sigma, df=95)[1]
p4_adj_MCPMod=1-pmvt(lower=rep(-Inf,7),upper =rep(t4,7), corr=Sigma, df=95)[1]
p5_adj_MCPMod=1-pmvt(lower=rep(-Inf,7),upper =rep(t5,7), corr=Sigma, df=95)[1]
p6_adj_MCPMod=1-pmvt(lower=rep(-Inf,7),upper =rep(t6,7), corr=Sigma, df=95)[1]
p7_adj_MCPMod=1-pmvt(lower=rep(-Inf,7),upper =rep(t7,7), corr=Sigma, df=95)[1]
p_adj_MCPMod=c(p1_adj_MCPMod,p2_adj_MCPMod,p3_adj_MCPMod,p4_adj_MCPMod,p5_adj_MCPMod,p6_adj_MCPMod,p7_adj_MCPMod)
round(p_adj_MCPMod,digits=4)

#########################################################################
## Adjusted p-values of Generalized multiple contrast tests in Table 1
## Applying the Closed testing procedure for all subset intersections
#########################################################################

#Indicator for all 2^7-1=127 subset intersections
Ind=matrix(0,nrow=7,ncol=127)
#col1 of Ind  Step1
combn(1:7,7)  
Ind[combn(1:7,7),1]=1
#col 2 to 8 of Ind Step2
choose(7, 6)
for (i in 1:choose(7, 6)){
   Ind[,(choose(7, 7)+1):(choose(7, 7)+choose(7, 6))][combn(1:7,6)[,i],i]=1
}
#col 9 to 29 of Ind Step3
choose(7, 5)
for (i in 1:choose(7, 5)){
   Ind[,(choose(7, 7)+choose(7, 6)+1):(choose(7, 7)+choose(7, 6)+choose(7, 5))][combn(1:7,5)[,i],i]=1
}
#col 30 to 64 of Ind Step4
choose(7, 4)
for (i in 1:choose(7, 4)){
   Ind[,(choose(7, 7)+choose(7, 6)+choose(7, 5)+1):
          (choose(7, 7)+choose(7, 6)+choose(7, 5)+choose(7, 4))][combn(1:7,4)[,i],i]=1
}
#col 65 to 99 of Ind Step5
choose(7, 3)
for (i in 1:choose(7, 3)){
   Ind[,(choose(7, 7)+choose(7, 6)+choose(7, 5)+choose(7, 4)+1):
          (choose(7, 7)+choose(7, 6)+choose(7, 5)+choose(7, 4)+choose(7, 3))][combn(1:7,3)[,i],i]=1
}
#col 100 to 120 of Ind Step6
choose(7, 2)
for (i in 1:choose(7, 2)){
   Ind[,(choose(7, 7)+choose(7, 6)+choose(7, 5)+choose(7, 4)+choose(7, 3)+1):
          (choose(7, 7)+choose(7, 6)+choose(7, 5)+choose(7, 4)+choose(7, 3)+choose(7, 2))][combn(1:7,2)[,i],i]=1
}
#col 121 to 127 of Ind Step7
choose(7, 1)
for (i in 1:choose(7, 1)){
   Ind[,(choose(7, 7)+choose(7, 6)+choose(7, 5)+choose(7, 4)+choose(7, 3)+choose(7, 2)+1):
          (choose(7, 7)+choose(7, 6)+choose(7, 5)+choose(7, 4)+choose(7, 3)+choose(7, 2)+choose(7, 1))][combn(1:7,1)[,i],i]=1
}

#Adjusted p-values of Tippett procedure in Table 1
allp_T=rep(NA,127)
allp_T[1]=min(p_adj_MCPMod)
set.seed(12345)
for (i in 2:120) {
   t_ind=(Ind[,i]*c(1,2,3,4,5,6,7))[Ind[,i]*c(1,2,3,4,5,6,7)>0]
   tt=(t*Ind[,i])[Ind[,i]>0]
   Sigmaa=Sigma[t_ind,t_ind]
   allp_T[i]=1-pmvt(lower=rep(-Inf,sum(Ind[,i])),upper =rep(max(tt),sum(Ind[,i])), corr=Sigmaa, df=df)[1]
}
allp_T[121:127]=p
#The adjusted p-value p_m, m=1,...,7 are the maximum p values for all subset intersection 
#hypotheses contain m
p1_adj_T=max(allp_T[Ind[1,]==1])
p2_adj_T=max(allp_T[Ind[2,]==1])
p3_adj_T=max(allp_T[Ind[3,]==1])
p4_adj_T=max(allp_T[Ind[4,]==1])
p5_adj_T=max(allp_T[Ind[5,]==1])
p6_adj_T=max(allp_T[Ind[6,]==1])
p7_adj_T=max(allp_T[Ind[7,]==1])
p_adj_T=c(p1_adj_T,p2_adj_T,p3_adj_T,p4_adj_T,p5_adj_T,p6_adj_T, p7_adj_T)
round(p_adj_T,digits=4)
   
#Adjusted p-values of Fisher procedure in Table 1
#Calculate the covariance terms using the covariance formula in
#Example 1: Fisher's combination method in the paper
cov_F12=3.279*rho12+0.711*rho12^2+0.727/df+0.129*rho12/df-0.768*rho12^2/df
cov_F13=3.279*rho13+0.711*rho13^2+0.727/df+0.129*rho13/df-0.768*rho13^2/df
cov_F14=3.279*rho14+0.711*rho14^2+0.727/df+0.129*rho14/df-0.768*rho14^2/df
cov_F15=3.279*rho15+0.711*rho15^2+0.727/df+0.129*rho15/df-0.768*rho15^2/df
cov_F16=3.279*rho16+0.711*rho16^2+0.727/df+0.129*rho16/df-0.768*rho16^2/df
cov_F17=3.279*rho17+0.711*rho17^2+0.727/df+0.129*rho17/df-0.768*rho17^2/df

cov_F23=3.279*rho23+0.711*rho23^2+0.727/df+0.129*rho23/df-0.768*rho23^2/df
cov_F24=3.279*rho24+0.711*rho24^2+0.727/df+0.129*rho24/df-0.768*rho24^2/df
cov_F25=3.279*rho25+0.711*rho25^2+0.727/df+0.129*rho25/df-0.768*rho25^2/df
cov_F26=3.279*rho26+0.711*rho26^2+0.727/df+0.129*rho26/df-0.768*rho26^2/df
cov_F27=3.279*rho27+0.711*rho27^2+0.727/df+0.129*rho27/df-0.768*rho27^2/df

cov_F34=3.279*rho34+0.711*rho34^2+0.727/df+0.129*rho34/df-0.768*rho34^2/df
cov_F35=3.279*rho35+0.711*rho35^2+0.727/df+0.129*rho35/df-0.768*rho35^2/df
cov_F36=3.279*rho36+0.711*rho36^2+0.727/df+0.129*rho36/df-0.768*rho36^2/df
cov_F37=3.279*rho37+0.711*rho37^2+0.727/df+0.129*rho37/df-0.768*rho37^2/df

cov_F45=3.279*rho45+0.711*rho45^2+0.727/df+0.129*rho45/df-0.768*rho45^2/df
cov_F46=3.279*rho46+0.711*rho46^2+0.727/df+0.129*rho46/df-0.768*rho46^2/df
cov_F47=3.279*rho47+0.711*rho47^2+0.727/df+0.129*rho47/df-0.768*rho47^2/df

cov_F56=3.279*rho56+0.711*rho56^2+0.727/df+0.129*rho56/df-0.768*rho56^2/df
cov_F57=3.279*rho57+0.711*rho57^2+0.727/df+0.129*rho57/df-0.768*rho57^2/df

cov_F67=3.279*rho67+0.711*rho67^2+0.727/df+0.129*rho67/df-0.768*rho67^2/df

cov_F=matrix(0, nrow=7, ncol=7)
cov_F[1,2]=cov_F12
cov_F[1,3]=cov_F13
cov_F[1,4]=cov_F14
cov_F[1,5]=cov_F15
cov_F[1,6]=cov_F16
cov_F[1,7]=cov_F17

cov_F[2,3]=cov_F23
cov_F[2,4]=cov_F24
cov_F[2,5]=cov_F25
cov_F[2,6]=cov_F26
cov_F[2,7]=cov_F27

cov_F[3,4]=cov_F34
cov_F[3,5]=cov_F35
cov_F[3,6]=cov_F36
cov_F[3,7]=cov_F37

cov_F[4,5]=cov_F45
cov_F[4,6]=cov_F46
cov_F[4,7]=cov_F47
cov_F[5,6]=cov_F56
cov_F[5,7]=cov_F57
cov_F[6,7]=cov_F67

allp_F=rep(NA,127)

for (i in 1:120) {
   p_ind=(Ind[,i]*c(1,2,3,4,5,6,7))[Ind[,i]*c(1,2,3,4,5,6,7)>0]
   psi_F_obs=-2*(sum(log((p*Ind[,i])[p*Ind[,i]>0])))
   E=2*sum(Ind[,i])
   sum=0
   for (d in 1:dim(combn(p_ind,2))[2] ) {
      sum=sum+cov_F[combn(p_ind,2)[1,d],combn(p_ind,2)[2,d]]
   }
   sum
   var=4*sum(Ind[,i])+2*sum
   f=2*E^2/var
   c=var/(2*E)
   allp_F[i]= 1-pchisq(psi_F_obs/c,df=f)
}

allp_F[121:127]=p

p1_adj=max(allp_F[Ind[1,]==1])
p2_adj=max(allp_F[Ind[2,]==1])
p3_adj=max(allp_F[Ind[3,]==1])
p4_adj=max(allp_F[Ind[4,]==1])
p5_adj=max(allp_F[Ind[5,]==1])
p6_adj=max(allp_F[Ind[6,]==1])
p7_adj=max(allp_F[Ind[7,]==1])
p_adj_F=c(p1_adj,p2_adj,p3_adj,p4_adj,p5_adj,p6_adj,p7_adj)
round(p_adj_F,digits=4)

#Adjusted p-values of Inverse normal procedure in Table 1
#Calculate the covariance terms using the covariance formula in
#Example 2: Inverse normal combination method in the paper
cov_N12=rho12+0.00019*rho12^3-0.02378*rho12/df+0.01891*rho12^3/df
cov_N13=rho13+0.00019*rho13^3-0.02378*rho13/df+0.01891*rho13^3/df
cov_N14=rho14+0.00019*rho14^3-0.02378*rho14/df+0.01891*rho14^3/df
cov_N15=rho15+0.00019*rho15^3-0.02378*rho15/df+0.01891*rho15^3/df
cov_N16=rho16+0.00019*rho16^3-0.02378*rho16/df+0.01891*rho16^3/df
cov_N17=rho17+0.00019*rho17^3-0.02378*rho17/df+0.01891*rho17^3/df

cov_N23=rho23+0.00019*rho23^3-0.02378*rho23/df+0.01891*rho23^3/df
cov_N24=rho24+0.00019*rho24^3-0.02378*rho24/df+0.01891*rho24^3/df
cov_N25=rho25+0.00019*rho25^3-0.02378*rho25/df+0.01891*rho25^3/df
cov_N26=rho26+0.00019*rho26^3-0.02378*rho26/df+0.01891*rho26^3/df
cov_N27=rho27+0.00019*rho27^3-0.02378*rho27/df+0.01891*rho27^3/df

cov_N34=rho34+0.00019*rho34^3-0.02378*rho34/df+0.01891*rho34^3/df
cov_N35=rho35+0.00019*rho35^3-0.02378*rho35/df+0.01891*rho35^3/df
cov_N36=rho36+0.00019*rho36^3-0.02378*rho36/df+0.01891*rho36^3/df
cov_N37=rho37+0.00019*rho37^3-0.02378*rho37/df+0.01891*rho37^3/df

cov_N45=rho45+0.00019*rho45^3-0.02378*rho45/df+0.01891*rho45^3/df
cov_N46=rho46+0.00019*rho46^3-0.02378*rho46/df+0.01891*rho46^3/df
cov_N47=rho47+0.00019*rho47^3-0.02378*rho47/df+0.01891*rho47^3/df

cov_N56=rho56+0.00019*rho56^3-0.02378*rho56/df+0.01891*rho56^3/df
cov_N57=rho57+0.00019*rho57^3-0.02378*rho57/df+0.01891*rho57^3/df

cov_N67=rho67+0.00019*rho67^3-0.02378*rho67/df+0.01891*rho67^3/df


cov_N=matrix(0, nrow=7, ncol=7)
cov_N[1,2]=cov_N12
cov_N[1,3]=cov_N13
cov_N[1,4]=cov_N14
cov_N[1,5]=cov_N15
cov_N[1,6]=cov_N16
cov_N[1,7]=cov_N17

cov_N[2,3]=cov_N23
cov_N[2,4]=cov_N24
cov_N[2,5]=cov_N25
cov_N[2,6]=cov_N26
cov_N[2,7]=cov_N27

cov_N[3,4]=cov_N34
cov_N[3,5]=cov_N35
cov_N[3,6]=cov_N36
cov_N[3,7]=cov_N37

cov_N[4,5]=cov_N45
cov_N[4,6]=cov_N46
cov_N[4,7]=cov_N47
cov_N[5,6]=cov_N56
cov_N[5,7]=cov_N57
cov_N[6,7]=cov_N67

allp_N=rep(NA,127)


for (i in 1:120) {
   p_ind=(Ind[,i]*c(1,2,3,4,5,6,7))[Ind[,i]*c(1,2,3,4,5,6,7)>0]
   psi_N_obs=1/sqrt(sum(Ind[,i]))*sum(qnorm(1-(p*Ind[,i])[p*Ind[,i]>0]))
   sum=0
   for (d in 1:dim(combn(p_ind,2))[2] ) {
      sum=sum+cov_N[combn(p_ind,2)[1,d],combn(p_ind,2)[2,d]]
   }
   sum
   var=1+2/sum(Ind[,i])*sum
   allp_N[i]= 1-pnorm(psi_N_obs,mean=0,sd=sqrt(var))
}

allp_N[121:127]=p
p1_adj=max(allp_N[Ind[1,]==1])
p2_adj=max(allp_N[Ind[2,]==1])
p3_adj=max(allp_N[Ind[3,]==1])
p4_adj=max(allp_N[Ind[4,]==1])
p5_adj=max(allp_N[Ind[5,]==1])
p6_adj=max(allp_N[Ind[6,]==1])
p7_adj=max(allp_N[Ind[7,]==1])
p_adj_N=c(p1_adj,p2_adj,p3_adj,p4_adj,p5_adj,p6_adj,p7_adj)
round(p_adj_N,digits=4)

#################################Model Selection######################################
### Select Emax model for all 4 procedures since it has the smallest adjusted p-value 
### fiting nonlinear Emax model
######################################################################################

resp=biom$resp
dose=biom$dose
fit=nls(resp ~ E0+Emax*dose/(ED50+dose),
      start = list(E0 = 0.2, Emax=0.7, ED50=0.2))
summary(fit)
coef(fit)

########################Dose selection ################################
### Calculate MED follow Section 3.2 in the paper
#######################################################################

gamma=0.05
n=20 #Group sample size
d0=seq(0,1,by=0.005)  
predict=coef(fit)[1]+coef(fit)[2]*dose/(dose+coef(fit)[3])
ss1=sqrt(sum((predict-resp)^2)/(5*n-3))
#The derivative matrix
V=cbind(rep(1,5*n),dose/(dose+coef(fit)[3]),
        -coef(fit)[2]*dose/(dose+coef(fit)[3])^2)
se=rep(NA,length(d0))
for (i in 1:length(d0)){
   v0=c(1, d0[i]/(d0[i]+coef(fit)[3]), -coef(fit)[2]*d0[i]/(d0[i]+coef(fit)[3])^2)
   se[i]=ss1*sqrt(t(v0)%*%solve(t(V)%*%V, tol=1e-24)%*%v0)
}
#The lower and upper 100(1-2*gamma)% confidence limits
Ld=coef(fit)[1]+coef(fit)[2]*d0/(d0+coef(fit)[3])-se*qt(1-gamma, n*5-3)
Ud=coef(fit)[1]+coef(fit)[2]*d0/(d0+coef(fit)[3])+se*qt(1-gamma, n*5-3)
#The mean response at dosage d0
pd=coef(fit)[1]+coef(fit)[2]*d0/(d0+coef(fit)[3])
#The mean response at placebo
pd1=coef(fit)[1]+coef(fit)[2]*0/(0+coef(fit)[3])
MED=min(d0[(Ld>pd1) & (pd>(pd1+0.4))])
MED #0.165